@@ -1,7 +1,3 @@
-
-#ifdef MAC_SSIZE
-typedef long ssize_t;
-#endif
 #include <stdlib.h>
 #include "EF_Util.h"
 
