@@ -62,7 +62,7 @@
 #include "gif/gif.h"
 
 #ifndef lint
-    static char rcsid[]	= "$Id: metafile.c 14748 2012-08-20 22:07:51Z ansley $";
+    static char rcsid[]	= "$Id$";
     static char afsid[]	= "$__Header$";
 #endif
 
@@ -1214,7 +1214,7 @@ XgksMiCloseWs(WS_STATE_PTR ws)
  * Close an output Metafile.  ?batmode
  */
     int
-XgksMoCloseWs(WS_STATE_PTR ws, int *batmode)
+XgksMoCloseWs(WS_STATE_PTR ws, int batmode)
 {
   return MO_CLOSE(&ws->mf);
 }
